package org.rcisoft.business.bcode.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.rcisoft.business.bcode.model.BCodeFile;
import org.rcisoft.business.bcode.service.BCodeService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

/**
 * Created by lcy on 18/1/3.
 *
 * 编程controller
 */
@RestController
@RequestMapping("/BCode")
public class BCodeController extends PaginationController {


    @Autowired
    private BCodeService bCodeServiceImpl;

    @Autowired
    private Global global;

    /**
     * 学生看自己的实验
     * @param slId
     * @param chapId
     * @param examType
     * @return
     */
    @ApiOperation(value="查看实验代码", notes="学生看自己的实验")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "examType", value = "类型 0001:JAVA单文件，0002:JAVA工程，1001:HTML单文件，1002:HTML工程", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryOwnExamFiles")
    public Result queryOwnExamFiles(@RequestParam("slId") String slId,
                                       @RequestParam("chapId") String chapId,
                                       @RequestParam("examType") String examType){
        List<BCodeFile> list = bCodeServiceImpl.readExamFiles(slId, chapId, examType,
                UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID),
                UserUtil.getUserInfoProp(getToken(), UserUtil.USER_USERNAME));
        if(examType.equals("0001")||examType.equals("1001"))
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    list.get(0));
        else
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    list);

    }



    @ApiOperation(value="保存实验代码", notes="学生保存自己的实验")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "content", value = "代码", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "examType", value = "类型 0001:JAVA单文件，0002:JAVA工程，1001:HTML单文件，1002:HTML工程", required = true, dataType = "varchar")})
    @PostMapping(value = "/saveOwnExamFiles")
    public Result saveOwnExamFiles(@RequestParam("slId") String slId,
                                   @RequestParam("chapId") String chapId,
                                   @RequestParam("examType") String examType,
                                   @RequestParam("content") String content){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCodeServiceImpl.saveOwnExamFiles(slId, chapId, examType, content,
                        UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID),
                        UserUtil.getUserInfoProp(getToken(), UserUtil.USER_USERNAME)
                        ));
    }


    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "开课编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "学生Id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "是否全部导出 1是全部导出  0否", required = true, dataType = "varchar")})
    @ApiOperation(value="导出学生代码文件", notes="根据学生Id导出学生代码文件")
    @GetMapping(value = "/exportFilesWithoutAuth")
    public void exportFilesWithoutAuth(String slId, String studentId, String type, String slCode, HttpServletResponse response)throws Exception{
                String fileName = bCodeServiceImpl.exportFiles(slId, studentId, type, slCode);
                this.downloadFiles(fileName,response);
    }


    public void downloadFiles(String fileName, HttpServletResponse response) throws Exception{
        ServletOutputStream out = null;
        InputStream in = null;
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getTEMP_LOCATION() + File.separator
                + global.getSTUDENT_CODE_LOCATION() + File.separator
                + fileName;
        try{
            File file = new File(path);
            if(!file.exists()){
                throw new ServiceException(ResultServiceEnums.DOWNLOAD_ERROR);
            }
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/force-download");
            response.addHeader("Content-Disposition","attachment;filename="+fileName);
            out = response.getOutputStream();
            in = new FileInputStream(file);
            byte[] buffer = new byte[1024];
            int bytesToRead = -1;
            while((bytesToRead = in.read(buffer)) != -1) {
                out.write(buffer,0, bytesToRead);
            }
        }catch (Exception ex){
            throw new ServiceException(ResultServiceEnums.DOWNLOAD_ERROR);
        }finally {
            if(in != null) {in.close();}
            if(out != null) {out.close();}
        }
    }



    @ApiImplicitParams({@ApiImplicitParam(name = "filePath", value = "文件路径", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "newFilePath", value = "新文件名（修改文件名时使用该参数）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "content", value = "内容（修改文件内容时使用该参数）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节Id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "fileType", value = "1新增文件夹，2新增文件，3修改文件名，4删除文件，5修改文件内容", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "codeType", value = "1单文件，2工程", required = false, dataType = "varchar")})
    @ApiOperation(value="单文件或工程各种操作", notes="单文件或工程各种操作")
    @GetMapping(value = "/createOrUpdateFiles")
    public Result createOrUpdateFiles(String filePath, String newFilePath, String content, String slId, String chapId, String fileType, String codeType) throws Exception{
        String codePath = "";
        if(codeType.equals("1")){
            codePath = global.getCOURSE_CODE_LOCATION();
        }else{
            codePath = global.getCOURSE_PROJECT_LOCATION();
        }
        String serverPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapId + File.separator
                + codePath + File.separator
                + UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID) + File.separator;
        File file = new File(serverPath+filePath);
        if(fileType.equals("1")&&!file.exists()){//新增文件夹
            file.mkdirs();
        }else if(fileType.equals("2")&&!file.exists()){//新增文件
            file.createNewFile();
        }else if(fileType.equals("3")&&file.exists()){//修改文件名
            File newFile = new File(newFilePath);
            file.renameTo(newFile);
        }else if(fileType.equals("4")&&file.exists()){//删除文件
            file.delete();
        }else if(fileType.equals("5")&&file.exists()){//修改文件内容
            FileUtils.write(file,content,"UTF-8");
        }
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "");
    }
}
